package com.temboo.Library.BrighterPlanet;

import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONException;
import java.math.BigDecimal;
import com.temboo.core.Choreography;
import com.temboo.core.Choreography.ResultSet;
import com.temboo.core.Choreography.InputSet;
import com.temboo.core.TembooException;
import com.temboo.core.TembooPath;
import com.temboo.core.TembooSession;

/** 
Flight

Returns information about the carbon footprint of a passenger's air travel.
*/
public class Flight extends Choreography {

	/**
	Create a new instance of the Flight Choreo. A TembooSession object, containing a valid
	set of Temboo credentials, must be supplied.
	*/
	public Flight(TembooSession session) {
		super(session, TembooPath.pathFromStringNoException("/Library/BrighterPlanet/Flight"));
	}


	/** 
	Obtain an InputSet object, used to define inputs for an execution of this Choreo.
	
	@return FlightInputSet
	*/
	public FlightInputSet newInputSet() {
		return new FlightInputSet();
	}
	
	
	/**
	 * Execute the Choreo using the specified InputSet as parameters, wait for the Choreo to complete 
	 * and return a ResultSet containing the execution results.
	 * @param choreoInputs
	 * @return
	 * @throws TembooException
	 */
	 @Override
	public FlightResultSet execute(InputSet choreoInputs) throws TembooException {
		JSONObject result = super.executeWithResults(choreoInputs);
		return new FlightResultSet(result);
	}
	
	
	/**
		An InputSet with methods appropriate for specifying the inputs to the Flight
        Choreo. The InputSet object is used to specify input parameters when executing this choreo.
	*/
	public static class FlightInputSet extends InputSet {
		/** 
		Set the value of the APIKey input for this Choreo. 

		@param String - (required, string) The API Key provided by Brighter Planet.
		*/
		public void set_APIKey(String value) {
			this.setInput("APIKey", value);
		}



		/** 
		Set the value of the Aircraft input for this Choreo. 

		@param String - (optional, string) The ICAO code associated with the aircraft.
		*/
		public void set_Aircraft(String value) {
			this.setInput("Aircraft", value);
		}



		/** 
		Set the value of the Airline input for this Choreo. 

		@param String - (optional, string) The name of the airline used (i.e. Continental, Delta, etc).
		*/
		public void set_Airline(String value) {
			this.setInput("Airline", value);
		}



		/** 
		Set the value of the Compliance input for this Choreo. 

		@param String - (optional, string) Comply with one or more protocols:  Greenhouse Gas Protocol Scope 3 (ghg_protocol_scope_3), ISO 14064-1 (iso), and The Climate Registry (tcr).
		*/
		public void set_Compliance(String value) {
			this.setInput("Compliance", value);
		}



		/** 
		Set the value of the Date input for this Choreo. 

		@param String - (optional, date) The date of the trip in YYYY-MM-DD format.
		*/
		public void set_Date(String value) {
			this.setInput("Date", value);
		}



		/** 
		Set the value of the DestinationAirport input for this Choreo. 

		@param String - (optional, string) The airport code for the destination (i.e. LGA, JFK, etc.).
		*/
		public void set_DestinationAirport(String value) {
			this.setInput("DestinationAirport", value);
		}



		/** 
		Set the value of the DistanceClass input for this Choreo. 

		@param String - (optional, string) The distance class associated it the air travel (i.e. long haul, short haul).
		*/
		public void set_DistanceClass(String value) {
			this.setInput("DistanceClass", value);
		}



		/** 
		Set the value of the DistanceEstimate input for this Choreo. 

		@param BigDecimal - (optional, decimal) A estimate of the distance of the travel in kilometres.
		*/
		public void set_DistanceEstimate(BigDecimal value) {
			this.setInput("DistanceEstimate", value);
		}

		/** 
		Set the value of the DistanceEstimate input for this Choreo as a String. 

		@param String - (optional, decimal) A estimate of the distance of the travel in kilometres.
		*/
		public void set_DistanceEstimate(String value) {
			this.setInput("DistanceEstimate", value);	
		}

		/** 
		Set the value of the Fuel input for this Choreo. 

		@param String - (optional, string) The type of fuel used in the aircraft (i.e. Aviation Gasoline, Biodiesel).
		*/
		public void set_Fuel(String value) {
			this.setInput("Fuel", value);
		}



		/** 
		Set the value of the OriginAirport input for this Choreo. 

		@param String - (optional, string) The airport code for the origin (i.e. LGA, JFK, etc.).
		*/
		public void set_OriginAirport(String value) {
			this.setInput("OriginAirport", value);
		}



		/** 
		Set the value of the ResponseFormat input for this Choreo. 

		@param String - (optional, string) Specify your desired response format. Accepted values are xml and json (the default).
		*/
		public void set_ResponseFormat(String value) {
			this.setInput("ResponseFormat", value);
		}



		/** 
		Set the value of the Seats input for this Choreo. 

		@param Integer - (optional, integer) The number of seats.
		*/
		public void set_Seats(Integer value) {
			this.setInput("Seats", value);
		}

		/** 
		Set the value of the Seats input for this Choreo as a String. 

		@param String - (optional, integer) The number of seats.
		*/
		public void set_Seats(String value) {
			this.setInput("Seats", value);	
		}

		/** 
		Set the value of the Timeframe input for this Choreo. 

		@param String - (optional, string) A date range specified in the following format: 2008-01-01/2008-07-09
		*/
		public void set_Timeframe(String value) {
			this.setInput("Timeframe", value);
		}



		/** 
		Set the value of the Trips input for this Choreo. 

		@param String - (optional, string) The number of trips to calculate information for.
		*/
		public void set_Trips(String value) {
			this.setInput("Trips", value);
		}



	}
	
	
	/**
		A ResultSet with methods tailored to the values returned by the Flight Choreo.
        The ResultSet object is used to retrieve the results of a Choreo execution.
	*/
	public static class FlightResultSet extends ResultSet {
		public FlightResultSet(JSONObject doc) throws TembooException {
			super(doc);
		}

		/** 
		Retrieve the value for the "Response" output from this Choreo execution

		@return String - The response from Brighter Planet.
		*/
		public String get_Response() {
			return this.getResult("Response");
		}
	}
	
}
