package com.temboo.Library.NYTimes.ArticleSearch;

import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONException;
import java.math.BigDecimal;
import com.temboo.core.Choreography;
import com.temboo.core.Choreography.ResultSet;
import com.temboo.core.Choreography.InputSet;
import com.temboo.core.TembooException;
import com.temboo.core.TembooPath;
import com.temboo.core.TembooSession;
import com.temboo.outputs.NYTimes.NYTimesArticle;

/** 
QueryArticles

Lets you query the Article Search API for New York Times articles.
*/
public class QueryArticles extends Choreography {

	/**
	Create a new instance of the QueryArticles Choreo. A TembooSession object, containing a valid
	set of Temboo credentials, must be supplied.
	*/
	public QueryArticles(TembooSession session) {
		super(session, TembooPath.pathFromStringNoException("/Library/NYTimes/ArticleSearch/QueryArticles"));
	}


	/** 
	Obtain an InputSet object, used to define inputs for an execution of this Choreo.
	
	@return QueryArticlesInputSet
	*/
	public QueryArticlesInputSet newInputSet() {
		return new QueryArticlesInputSet();
	}
	
	
	/**
	 * Execute the Choreo using the specified InputSet as parameters, wait for the Choreo to complete 
	 * and return a ResultSet containing the execution results.
	 * @param choreoInputs
	 * @return
	 * @throws TembooException
	 */
	 @Override
	public QueryArticlesResultSet execute(InputSet choreoInputs) throws TembooException {
		JSONObject result = super.executeWithResults(choreoInputs);
		return new QueryArticlesResultSet(result);
	}
	
	
	/**
		An InputSet with methods appropriate for specifying the inputs to the QueryArticles
        Choreo. The InputSet object is used to specify input parameters when executing this choreo.
	*/
	public static class QueryArticlesInputSet extends InputSet {
		/** 
		Set the value of the APIKey input for this Choreo. 

		@param String - (required, string) The API Key provided by NY Times.
		*/
		public void set_APIKey(String value) {
			this.setInput("APIKey", value);
		}



		/** 
		Set the value of the BeginDate input for this Choreo. 

		@param String - (optional, date) Sets the starting point (which is inclusive) of the range of publication dates to return. Must be used with EndDate. Date should be formatted like YYYYMMDD.
		*/
		public void set_BeginDate(String value) {
			this.setInput("BeginDate", value);
		}



		/** 
		Set the value of the EndDate input for this Choreo. 

		@param String - (optional, date) Sets the end point (which is inclusive) of the range of publication dates to return. Must be used with BeginDate. Date should be formatted like YYYYMMDD.
		*/
		public void set_EndDate(String value) {
			this.setInput("EndDate", value);
		}



		/** 
		Set the value of the Facets input for this Choreo. 

		@param String - (optional, string) A comma-delimited list of up to 5 facets. This indicates the sets of facet values to include in the response. See Choreo documentation for more information about accepted values for this input.
		*/
		public void set_Facets(String value) {
			this.setInput("Facets", value);
		}



		/** 
		Set the value of the Fields input for this Choreo. 

		@param String - (optional, string) A comma-delimited list of fields to return. These fields are returned by default: body, byline, date, title, and url.
		*/
		public void set_Fields(String value) {
			this.setInput("Fields", value);
		}



		/** 
		Set the value of the Offset input for this Choreo. 

		@param Integer - (optional, integer) This corresponds to which set of 10 results is returned. Used to page through results. Set to 0 to return records 0-9, set to 1 to return records 10-19, etc.
		*/
		public void set_Offset(Integer value) {
			this.setInput("Offset", value);
		}

		/** 
		Set the value of the Offset input for this Choreo as a String. 

		@param String - (optional, integer) This corresponds to which set of 10 results is returned. Used to page through results. Set to 0 to return records 0-9, set to 1 to return records 10-19, etc.
		*/
		public void set_Offset(String value) {
			this.setInput("Offset", value);	
		}

		/** 
		Set the value of the Query input for this Choreo. 

		@param String - (required, string) Search keywords (optionally applied to specific fields) and/or facets. See Choreo documentation for syntax examples.
		*/
		public void set_Query(String value) {
			this.setInput("Query", value);
		}



		/** 
		Set the value of the Rank input for this Choreo. 

		@param String - (optional, string) Sets the order of the results. Accepted values are: newest (the defaults), oldest, or closest.
		*/
		public void set_Rank(String value) {
			this.setInput("Rank", value);
		}



	}
	
	
	/**
		A ResultSet with methods tailored to the values returned by the QueryArticles Choreo.
        The ResultSet object is used to retrieve the results of a Choreo execution.
	*/
	public static class QueryArticlesResultSet extends ResultSet {
		public QueryArticlesResultSet(JSONObject doc) throws TembooException {
			super(doc);
		}

		/** 
		Retrieve the value for the "Response" output from this Choreo execution

		@return String - (json) The response from the NY Times API.
		*/
		public String get_Response() {
			return this.getResult("Response");
		}
		/** The value of offset corresponds to a set of 10 results (i.e. offset=0 for results 0-9, offset=1 for results 10-19, etc)
		*/
		public String getOffset() {
		    return (String) ((JSONObject)getJSONFromString(this.getResult("Response"))).opt("offset");
		}
		/** Get an article matching the search criteria
		*/
		public NYTimesArticle[] getArticles() {
		    JSONArray list = (JSONArray) ((JSONObject)getJSONFromString(this.getResult("Response"))).opt("results");
		    NYTimesArticle[] resultArray = new NYTimesArticle[(list != null) ? list.length() : 0];
		    if(list != null) {
		    	for(int i=0; i<list.length(); i++) {
		    	    try {
		    	        resultArray[i] = new NYTimesArticle(list.get(i));
		    	    } catch (JSONException e) {}
		    	}
		    }
		    return resultArray;

		}
		/** Contains the search tokens provided for the query
		*/
		public String[] getTokens() {
		    JSONArray list = (JSONArray) ((JSONObject)getJSONFromString(this.getResult("Response"))).opt("tokens");
		    String[] resultArray = new String[(list != null) ? list.length() : 0];
		    if(list != null) {
		    	for(int i=0; i<list.length(); i++) {
		    	    try {
		    	        resultArray[i] = (String) list.get(i);
		    	    } catch (JSONException e) {}
		    	}
		    }
		    return resultArray;

		}
		/** The total number of articles for this search
		*/
		public Number getTotal() {
		    return (Number) ((JSONObject)getJSONFromString(this.getResult("Response"))).opt("total");
		}
	}
	
}
