package com.temboo.Library.SchoolFinder;

import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONException;
import java.math.BigDecimal;
import com.temboo.core.Choreography;
import com.temboo.core.Choreography.ResultSet;
import com.temboo.core.Choreography.InputSet;
import com.temboo.core.TembooException;
import com.temboo.core.TembooPath;
import com.temboo.core.TembooSession;

/** 
SchoolSearch

Returns a list of school district profiles and related school information for a zip code, city, state, or other criteria in the search parameters.
*/
public class SchoolSearch extends Choreography {

	/**
	Create a new instance of the SchoolSearch Choreo. A TembooSession object, containing a valid
	set of Temboo credentials, must be supplied.
	*/
	public SchoolSearch(TembooSession session) {
		super(session, TembooPath.pathFromStringNoException("/Library/SchoolFinder/SchoolSearch"));
	}


	/** 
	Obtain an InputSet object, used to define inputs for an execution of this Choreo.
	
	@return SchoolSearchInputSet
	*/
	public SchoolSearchInputSet newInputSet() {
		return new SchoolSearchInputSet();
	}
	
	
	/**
	 * Execute the Choreo using the specified InputSet as parameters, wait for the Choreo to complete 
	 * and return a ResultSet containing the execution results.
	 * @param choreoInputs
	 * @return
	 * @throws TembooException
	 */
	 @Override
	public SchoolSearchResultSet execute(InputSet choreoInputs) throws TembooException {
		JSONObject result = super.executeWithResults(choreoInputs);
		return new SchoolSearchResultSet(result);
	}
	
	
	/**
		An InputSet with methods appropriate for specifying the inputs to the SchoolSearch
        Choreo. The InputSet object is used to specify input parameters when executing this choreo.
	*/
	public static class SchoolSearchInputSet extends InputSet {
		/** 
		Set the value of the APIKey input for this Choreo. 

		@param String - (required, string) Your School Finder API Key.
		*/
		public void set_APIKey(String value) {
			this.setInput("APIKey", value);
		}



		/** 
		Set the value of the City input for this Choreo. 

		@param String - (conditional, string) The name of a city. Must also be accompanied by the corresponding state parameter.
		*/
		public void set_City(String value) {
			this.setInput("City", value);
		}



		/** 
		Set the value of the County input for this Choreo. 

		@param String - (conditional, string) The name of a county.
		*/
		public void set_County(String value) {
			this.setInput("County", value);
		}



		/** 
		Set the value of the Distance input for this Choreo. 

		@param BigDecimal - (conditional, decimal) A distance in miles from a specific latitude/longitude. The suggested value is around 1.5 miles. Please note that distance is required when using latitude and longitude parameters.
		*/
		public void set_Distance(BigDecimal value) {
			this.setInput("Distance", value);
		}

		/** 
		Set the value of the Distance input for this Choreo as a String. 

		@param String - (conditional, decimal) A distance in miles from a specific latitude/longitude. The suggested value is around 1.5 miles. Please note that distance is required when using latitude and longitude parameters.
		*/
		public void set_Distance(String value) {
			this.setInput("Distance", value);	
		}

		/** 
		Set the value of the DistrictID input for this Choreo. 

		@param String - (optional, string) The internal Education.com id of a school district.
		*/
		public void set_DistrictID(String value) {
			this.setInput("DistrictID", value);
		}



		/** 
		Set the value of the DistrictLEA input for this Choreo. 

		@param String - (optional, string) The NCES Local Education Agency (LEA) id of a school district.
		*/
		public void set_DistrictLEA(String value) {
			this.setInput("DistrictLEA", value);
		}



		/** 
		Set the value of the Latitude input for this Choreo. 

		@param BigDecimal - (conditional, decimal) A latitude which serves as the center for distance searches. Please note that distance is required when using latitude and longitude parameters.
		*/
		public void set_Latitude(BigDecimal value) {
			this.setInput("Latitude", value);
		}

		/** 
		Set the value of the Latitude input for this Choreo as a String. 

		@param String - (conditional, decimal) A latitude which serves as the center for distance searches. Please note that distance is required when using latitude and longitude parameters.
		*/
		public void set_Latitude(String value) {
			this.setInput("Latitude", value);	
		}

		/** 
		Set the value of the Longitude input for this Choreo. 

		@param BigDecimal - (conditional, decimal) A longitude which serves as the center for distance searches. Please note that distance is required when using latitude and longitude parameters.
		*/
		public void set_Longitude(BigDecimal value) {
			this.setInput("Longitude", value);
		}

		/** 
		Set the value of the Longitude input for this Choreo as a String. 

		@param String - (conditional, decimal) A longitude which serves as the center for distance searches. Please note that distance is required when using latitude and longitude parameters.
		*/
		public void set_Longitude(String value) {
			this.setInput("Longitude", value);	
		}

		/** 
		Set the value of the MinResult input for this Choreo. 

		@param BigDecimal - (optional, decimal) Minimum number of search results to return. The search will be expanded in increments of 0.5 miles until the minresult is reached. minResult is only valid for zip code and latitude/longitude requests.
		*/
		public void set_MinResult(BigDecimal value) {
			this.setInput("MinResult", value);
		}

		/** 
		Set the value of the MinResult input for this Choreo as a String. 

		@param String - (optional, decimal) Minimum number of search results to return. The search will be expanded in increments of 0.5 miles until the minresult is reached. minResult is only valid for zip code and latitude/longitude requests.
		*/
		public void set_MinResult(String value) {
			this.setInput("MinResult", value);	
		}

		/** 
		Set the value of the NCES input for this Choreo. 

		@param String - (optional, string) The National Center for Education Statistics (NCES) id of the school you want to find.
		*/
		public void set_NCES(String value) {
			this.setInput("NCES", value);
		}



		/** 
		Set the value of the ResponseFormat input for this Choreo. 

		@param String - (optional, string) Format of the response returned by Education.com. Defaluts to XML. JSON is also possible.
		*/
		public void set_ResponseFormat(String value) {
			this.setInput("ResponseFormat", value);
		}



		/** 
		Set the value of the SchoolID input for this Choreo. 

		@param String - (optional, string) The Education.com id of the school you want to find.
		*/
		public void set_SchoolID(String value) {
			this.setInput("SchoolID", value);
		}



		/** 
		Set the value of the State input for this Choreo. 

		@param String - (conditional, string) The two letter abbreviation of a state e.g. South Caroline should be formatted “SC”.
		*/
		public void set_State(String value) {
			this.setInput("State", value);
		}



		/** 
		Set the value of the Zip input for this Choreo. 

		@param Integer - (conditional, integer) A five digit US postal code.
		*/
		public void set_Zip(Integer value) {
			this.setInput("Zip", value);
		}

		/** 
		Set the value of the Zip input for this Choreo as a String. 

		@param String - (conditional, integer) A five digit US postal code.
		*/
		public void set_Zip(String value) {
			this.setInput("Zip", value);	
		}

	}
	
	
	/**
		A ResultSet with methods tailored to the values returned by the SchoolSearch Choreo.
        The ResultSet object is used to retrieve the results of a Choreo execution.
	*/
	public static class SchoolSearchResultSet extends ResultSet {
		public SchoolSearchResultSet(JSONObject doc) throws TembooException {
			super(doc);
		}

		/** 
		Retrieve the value for the "Response" output from this Choreo execution

		@return String - The response from Education.com.
		*/
		public String get_Response() {
			return this.getResult("Response");
		}
	}
	
}
