package com.temboo.outputs.Stripe;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONException;
import com.temboo.outputs.Stripe.StripeLines;

/**   Invoices are statements of what a customer owes for a particular billing period, including subscriptions, invoice items, and any automatic proration adjustments if necessary. Once an invoice is created, payment is automatically attempted. Note that the payment, while automatic, does not happen exactly at the time of invoice creation. If you have configured webhooks, the invoice will wait until one hour after the last webhook is successfully sent (or the last webhook times out after failing). Any customer credit on the account is applied before determining how much is due for that invoice (the amount that will be actually charged). If the amount due for the invoice is less than 50 cents (the minimum for a charge), we add the amount to the customer's running account balance to be added to the next invoice. If this amount is negative, it will act as a credit to offset the next invoice. Note that the customer account balance does not include unpaid invoices; it only includes balances that need to be taken into account when calculating the amount due for the next invoice.

*/
public class StripeInvoice {
    protected JSONObject base;
    
    public StripeInvoice(Object base) {
    	if (base instanceof JSONObject) {
            this.base = (JSONObject) base;
        } else if (base instanceof JSONArray) {
            if (((JSONArray)base).length()==0) {
                this.base=new JSONObject();
            } else {
                throw new ClassCastException("Expecting JSONObject but got non-empty JSONArray");
            }
        }
    }

    /** Final amount due at this time for this invoice. If the invoice's total is smaller than the minimum charge amount, for example, or if there is account credit that can be applied to the invoice, the amount_due may be 0. If there is a positive starting_balance for the invoice (the customer owes money), the amount_due will also take that into account. The charge that gets generated for the invoice will be for the amount specified in amount_due.
    */
    public Number getAmountDue() {
        return (Number) base.opt("amount_due");
    }

    /** Number of automatic payment attempts made for this invoice. Does not include manual attempts to pay the invoice.
    */
    public Number getAttemptCount() {
        return (Number) base.opt("attempt_count");
    }

    /** Whether or not an attempt has been made to pay the invoice. An invoice is not attempted until 1 hour after the invoice.created webhook, for example, so you might not want to display that invoice as unpaid to your users.
    */
    public Boolean getAttempted() {
        return (Boolean) base.opt("attempted");
    }

    /** ID of the charge
    */
    public String getCharge() {
        return (String) base.opt("charge");
    }

    /** Whether or not the invoice is still trying to collect payment. An invoice is closed if it's either paid or it has been marked closed. A closed invoice will no longer attempt to collect payment.
    */
    public Boolean getClosed() {
        return (Boolean) base.opt("closed");
    }

    /** (3-letter ISO currency code)
    */
    public String getCurrency() {
        return (String) base.opt("currency");
    }

    /** ID of the customer
    */
    public String getCustomer() {
        return (String) base.opt("customer");
    }

    /** timestamp
    */
    public Number getDate() {
        return (Number) base.opt("date");
    }

    /** Ending customer balance after attempting to pay invoice. If the invoice has not been attempted yet, this will be nu
    */
    public Number getEndingBalance() {
        return (Number) base.opt("ending_balance");
    }

    /** ID of the Invoice
    */
    public String getId() {
        return (String) base.opt("id");
    }

    /** 
    */
    public StripeLines getLines() {
        return new StripeLines(base.opt("lines"));
    }

    /** In livemode, credit card transactions are actually processed.  In testmode, transactions don't go through the credit card network.
    */
    public Boolean getLivemode() {
        return (Boolean) base.opt("livemode");
    }

    /** timestamp
    */
    public Number getNextPaymentAttempt() {
        return (Number) base.opt("next_payment_attempt");
    }

    /** Value is always "invoice"
    */
    public String getObject() {
        return (String) base.opt("object");
    }

    /** Whether or not payment was successfully collected for this invoice. An invoice can be paid (most commonly) with a charge or with credit from the customer's account balance.
    */
    public Boolean getPaid() {
        return (Boolean) base.opt("paid");
    }

    /** timestamp.  End of the usage period the invoice covers
    */
    public Number getPeriodEnd() {
        return (Number) base.opt("period_end");
    }

    /** timestamp.  Start of the usage period the invoice covers
    */
    public Number getPeriodStart() {
        return (Number) base.opt("period_start");
    }

    /** Starting customer balance before attempting to pay invoice. If the invoice has not been attempted yet, this will be the current customer balance.
    */
    public Number getStartingBalance() {
        return (Number) base.opt("starting_balance");
    }

    /** Total of all subscriptions, invoice items, and prorations on the invoice before any discount is applied
    */
    public Number getSubtotal() {
        return (Number) base.opt("subtotal");
    }

    /** Total after discount
    */
    public Number getTotal() {
        return (Number) base.opt("total");
    }
    
    /* Internal utility method; retrieve the base JSON object for this element of the response data.
    */
    public JSONObject getBase() {
        return base;
    }
	/* Internal utility method; retrieve a sub-object from a JSON object/array; returns an empty object if key is not present
	*/
	protected JSONObject getJSONObject(Object json, String key) {
		JSONObject toReturn = null;

		try {
			if(json != null && (json instanceof JSONObject))
				toReturn = ((JSONObject) json).optJSONObject(key);					
		} catch(Exception e) {
		}
		if(toReturn == null)
			toReturn = new JSONObject();
		
		return toReturn;
	}
}